@@ -328,8 +328,10 @@ static x_cfg *our_cconfig(const conn_rec *c)
 static void example_log_each(apr_pool_t *p, server_rec *s, const char *note)
 {
     if (s != NULL) {
-        ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s, "mod_example_hooks: %s", note);
-    } else {
+        ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s, APLOGNO(02991)
+                     "mod_example_hooks: %s", note);
+    }
+    else {
         apr_file_t *out = NULL;
         apr_file_open_stderr(&out, p);
         apr_file_printf(out, "mod_example_hooks traced in non-loggable "
@@ -740,8 +742,7 @@ static int x_pre_config(apr_pool_t *pconf, apr_pool_t *plog,
     /*
      * Log the call and exit.
      */
-    trace_startup(ptemp, NULL, NULL, "x_pre_config()");
-
+    trace_startup(pconf, NULL, NULL, "x_pre_config()");
     return OK;
 }
 
@@ -762,7 +763,7 @@ static int x_check_config(apr_pool_t *pconf, apr_pool_t *plog,
     /*
      * Log the call and exit.
      */
-    trace_startup(ptemp, s, NULL, "x_check_config()");
+    trace_startup(pconf, s, NULL, "x_check_config()");
     return OK;
 }
 
@@ -799,7 +800,7 @@ static int x_open_logs(apr_pool_t *pconf, apr_pool_t *plog,
     /*
      * Log the call and exit.
      */
-    trace_startup(ptemp, s, NULL, "x_open_logs()");
+    trace_startup(pconf, s, NULL, "x_open_logs()");
     return OK;
 }
 
@@ -819,7 +820,7 @@ static int x_post_config(apr_pool_t *pconf, apr_pool_t *plog,
     /*
      * Log the call and exit.
      */
-    trace_startup(ptemp, s, NULL, "x_post_config()");
+    trace_startup(pconf, s, NULL, "x_post_config()");
     return OK;
 }
 
@@ -1049,7 +1050,8 @@ static int x_handler(request_rec *r)
                                     r->connection->pool);
     if ((status == APR_SUCCESS) && conn_data) {
         ap_rprintf(r, "  <OL>\n%s  </OL>\n", (char *) conn_data);
-    } else {
+    }
+    else {
         ap_rputs("  <P>No connection-specific callback information was "
                  "retrieved.</P>\n", r);
     }
@@ -1135,7 +1137,6 @@ static int x_pre_connection(conn_rec *c, void *csd)
 static int x_process_connection(conn_rec *c)
 {
     trace_connection(c, "x_process_connection()");
-
     return DECLINED;
 }
 
@@ -1181,7 +1182,6 @@ static int x_post_read_request(request_rec *r)
  */
 static int x_translate_name(request_rec *r)
 {
-
     /*
      * We don't actually *do* anything here, except note the fact that we were
      * called.
@@ -1448,6 +1448,7 @@ static int x_monitor(apr_pool_t *p, server_rec *s)
  */
 static void x_register_hooks(apr_pool_t *p)
 {
+    trace = NULL;
     ap_hook_pre_config(x_pre_config, NULL, NULL, APR_HOOK_MIDDLE);
     ap_hook_check_config(x_check_config, NULL, NULL, APR_HOOK_MIDDLE);
     ap_hook_test_config(x_test_config, NULL, NULL, APR_HOOK_MIDDLE);
